struct Char
{
    unsigned char tile;    //0-FF
    unsigned char palette; //0-3
public:
    bool operator== (const Char& b) const
    {
        return tile==b.tile && palette==b.palette;
    }
};

struct Block
{
    Char c[4];
public:
    void Write(unsigned offs, unsigned color_offs) const
    {
        for(unsigned a=0; a<4; ++a)
            ROM[offs + a] = c[a].tile;
            
        unsigned palette =
            (c[0].palette<<0) |
            (c[2].palette<<2) |
            (c[1].palette<<4) |
            (c[3].palette<<6);
            
        ROM[color_offs] = palette;
    }
    void Load(unsigned offs, unsigned color_offs)
    {
        unsigned char palettebyte = ROM[color_offs];
        for(unsigned b=0; b<4; ++b)
        {
            static const unsigned trans[4] = {0,2,1,3};
            unsigned transt = trans[b];
            
            char bit1 = 1&(palettebyte >> ((transt*2) +1));
            char bit2 = 1&(palettebyte >> ((transt*2) +0));
            
            c[b].palette = bit1*2 + bit2;
            c[b].tile    = ROM[offs + b];
        }
    }
    
    bool operator==(const Block& b) const
    {
        return c[0] == b.c[0]
            && c[1] == b.c[1]
            && c[2] == b.c[2]
            && c[3] == b.c[3];
    }
};

struct Room
{
    unsigned blocks[8*8]; // blockid
public:
    void Write(unsigned ptroffs, unsigned offs) const
    {
        ROM[ptroffs+0] = ((offs+0x8000) & 255);
        ROM[ptroffs+1] = ((offs+0x8000) >> 8);
        for(unsigned a=0; a<64; ++a) ROM[offs+a] = blocks[a];
    }
    
    void Load(unsigned ptroffs)
    {
        unsigned offs = ROM[ptroffs+0] | (ROM[ptroffs+1] << 8);
        offs = (ptroffs & 0x1C000) | (offs & 0x3FFF);
        for(unsigned a=0; a<64; ++a) blocks[a] = ROM[offs+a];
    }
    
    bool operator==(const Room& b) const
    {
        for(unsigned a=0; a<64; ++a)
            if(blocks[a] != b.blocks[a])
                return false;
        return true;
    }
};

struct Active
{
    unsigned char type;
    unsigned char screen;
    unsigned char x1,y1, x2,y2;
    unsigned char counter; // used for ordering buzz-blocks
    
    Active() : screen(255), counter(0) { }
    Active(int t,int s,int x,int y,int X,int Y):
      type(t),screen(s),x1(x),y1(y),x2(X),y2(Y), counter(0) { }
 
    void Delete() { screen=255; }
 
    bool operator< (const Active& b) const
    {
        if(screen != b.screen) return screen < b.screen;
        if(type != b.type) return type < b.type;
        if(counter != b.counter) return counter < b.counter;
        return x1 < b.x1;
    }
};

struct Enemy
{
    unsigned char screen;
    unsigned char x,y;
    unsigned char type;
    
    Enemy() : screen(255) { }
    Enemy(int s,int X,int Y,int t):
      screen(s),x(X),y(Y),type(t) { }

    void Delete() { screen=255; }
 
    bool operator< (const Enemy& b) const
    {
        if(screen != b.screen) return screen < b.screen;
        if(y != b.y) return y < b.y;
        return x < b.x;
    }
};

struct Screen
{
    char ascii[32*16+1];
    int cont_dir;
        
    Char outmap[16*16];
};
