static void CutmanAscii(Screen& s)
{
    for(unsigned y=0; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
        {
            unsigned char c = s.outmap[y*16+x].tile;
            char ch = '?';
            /* Classify types.
             *
             *
             *
             */
            switch(c)
            {
                // air:
                case 0x00:
                case 0x01:
                default:
                    ch = '.';
                    break;
                case 0x23:
                case 0x24:
                case 0x25:
                case 0x26:
                case 0x2B:
                case 0x2C:
                case 0x34:
                case 0x35:
                case 0x3C:
                    ch = ' ';
                    break;

                case 0x62:
                case 0x68:
                case 0x69: // all kinds of walls
                case 0x51: case 0x52: case 0x53: // wily signature
                    ch = '#';
                    break;
                case 0x38:
                case 0x39:
                case 0x3A:
                case 0x3B:
                    ch = Ascii_Guts;
                    break;
                case 0x82:
                    ch = Ascii_Ladder;
                    break;
                case 0xC4:
                    ch = Ascii_Spike;
                    break;

                case 0x10:
                    ch = Ascii_WilyDoor;
                    break;
            }
            
            s.ascii[y*32+x] = ch;
            s.ascii[y*32+x+16] = ' ';
            //printf("%02X",c);
        }
}
static void CutmanChar(Screen& s)
{
    const char* inmap = s.ascii;
    Char* outmap = s.outmap;

    char decorations[16*16];
    for(unsigned y=0; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
        {
            char c;
            switch(std::rand() % 6)
            {
                case 0: c = 1; break;
                case 1: c = 2; break;
                default: c = 0; break;
            }
            decorations[y*16+x] = c;
        }


    // Convert to tiles
    for(unsigned y=0; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
        {
            unsigned char c   = 0;
            unsigned char pal = 0;
            
            /* Cutman palettes:
             *   
             * 0 = solid
             * 1 = blacks
             * 2 = blues
             * 3 = blue,gray
             */
            
            switch(inmap[y*16+x])
            {
                case '.':
                    c   = 0x01; // background
                    pal = 0x01;
                    break;
                case ' ':
                    if(y > 0 &&
                        (inmap[(y-1)*16+x] != '.'
                      && inmap[(y-1)*16+x] != ' ')
                      )
                        c = 0x25; // shadow
                    else
                    {
                        switch(decorations[y*16+x])
                        {
                            case 1:
                                c = 0x2C; // inset decoration
                                break;
                            case 2:
                                c = 0x35;
                                if(x<16
                                && decorations[y*16+x+1]==2
                                && (x==0 || decorations[y*16+x-1]!=2))
                                    c = 0x34; // beginning

                                if(x>0  && inmap[y*16+x-1] == ' '
                                && x<15 && inmap[y*16+x+1] == ' ')
                                {
                                    break;
                                }
                                // passthru
                            case 0:
                                c = 0x26; // no decoration
                        }
                    }
                    pal = 0x02;
                    break;
                case '#':
                    if(y > 0 &&
                        (inmap[(y-1)*16+x] != '#')
                      )
                        c = 0x62; // step
                    else
                        c = 0x68 + ((x+y) & 1);
                    pal = 0x00;
                    break;
                case Ascii_Ladder:
                    c   = 0x82; // ladder
                    pal = 0x00;
                    break;
                case Ascii_Spike:
                    c   = 0xC4; // spike
                    if(y>0 && inmap[(y-1)*16+x] == ' ')
                        pal = 0x03;
                    else
                        pal = 0x01;
                    break;
                case Ascii_Guts:
                    c   = 0x38 + (x&1) + (y&1)*2; // gutsblock
                    pal = 0x00;
                    break;
                case Ascii_WilyDoor:
                    c   = 0x10;
                    pal = 0x00;
                    break;
                default:
                    printf("? '%c'\n", inmap[y*16+x]);
            }
            
            outmap[y*16+x].tile    = c;
            outmap[y*16+x].palette = pal;
        }
}

static Level Cutman =
{
    0, // level number,
    "Cutman",

    // screens:
    {
        {
        ". .............."
        ". .............."
        ". .............."
        ". .............."
        ". ......GG......"
        ". ......GG......"
        "# ###|####|#..##"
        ". ...|....|....."
        ". ...|....GG...."
        ". ...|....GG...."
        ". ...|..GGGG...."
        ". ...|..GGGG...."
        "# ##########  ##"
        "# ##########  ##"
        "# ##########  ##"
        "# ##########  ##"
        ,Shutter + Down + Up},
        {
        ". .............."
        ". .............."
        ". .............."
        ". .............."
        ". ......GG......"
        ". ......GG......"
        "# ###|####|#..##"
        ". ...|....|....."
        ". ...|....GG...."
        ". ...|....GG...."
        ". ...|..GGGG...."
        ". ...|..GGGG...."
        "  ##############"
        "  ##############"
        "  ##############"
        "  ##############"
        ,Right},
        {
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "........GG......"
        ".##.....GG......"
        "......####......"
        "......####..##.."
        "....######..##.."
        "....######..####"
        "##########..####"
        "##########..####"
        "##########..####"
        "##########..####"
        ,Right},
        {
        "............#|##"
        "............#|##"
        ".............|##"
        ".............|##"
        "....#|#..#######"
        "....#|#..#######"
        "....#|......####"
        "....#|......####"
        "....#|......####"
        "....#|###...####"
        "...........#####"
        "...........#####"
        "################"
        "################"
        "################"
        "################"
        ,Up},
        {
        ".............|##"
        ".............|##"
        ".............|##"
        "..........######"
        "..........######"
        "........########"
        "........########"
        "......##########"
        "..|#..##########"
        "..|#..........##"
        "..|...........##"
        "..|...........##"
        "..|...........##"
        "##########..#|##"
        "##########..#|##"
        "##########..#|##"
        ,Up},
        {
        ".............|##"
        ".............|##"
        ".............|##"
        "............####"
        "............####"
        "..|#......######"
        "..|#......######"
        "..|.....#|######"
        "..|.....#|######"
        "..|......|....##"
        "..|......|....##"
        "..##.....|....##"
        "..##.....|....##"
        "..###...#####|##"
        "..###...#####|##"
        "..###...#####|##"
        ,Up},
        {
        "...|..##########"
        "...|...#########"
        "...|......######"
        "...|.......#####"
        "...|........####"
        "...#####....####"
        "............####"
        "............####"
        "...#|###########"
        "....|.......####"
        "....|.........##"
        "....|.........##"
        "....|.........##"
        "...####..####|##"
        "...####..####|##"
        "...####..####|##"
        ,Up},
        {
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "............####"
        "...|############"
        "...|############"
        "...|############"
        ,Right},
        {
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "........GG......"
        ".##.....GG......"
        "......####......"
        "......####..##.."
        "....######..##.."
        "....######..####"
        "##########..####"
        "##########..####"
        "##########..####"
        "##########..####"
        ,Right},
        {
        "............#|##"
        "............#|##"
        ".............|##"
        ".............|##"
        "....#|#..#######"
        "....#|#..#######"
        "....#|......####"
        "....#|......####"
        "....#|......####"
        "....#|###...####"
        "...........#####"
        "...........#####"
        "################"
        "################"
        "################"
        "################"
        ,Up},
        {
        "##...........|##"
        "##...........|##"
        "##...........|##"
        "##...........|##"
        "####........####"
        "##............##"
        "##......##....##"
        "####|#......####"
        "##..|.........##"
        "##..|.........##"
        "##..|.GG......##"
        "##..|.GG......##"
        "#############|##"
        "#############|##"
        "#############|##"
        "#############|##"
        ,Up},
        {
        "##|...........##"
        "##|...........##"
        "##|...........##"
        "##|...........##"
        "##|...........##"
        "##|...........##"
        "##|.........####"
        "##|.........####"
        "##GG......######"
        "##GG......######"
        "######..########"
        "######........##"
        "######........##"
        "#############|##"
        "#############|##"
        "#############|##"
        ,Up},
        {
        "###|############"
        "###|############"
        "##.|..........##"
        "##.|..........##"
        "##............##"
        "##............##"
        "####|.......####"
        "##..|.......####"
        "##..|.GG....####"
        "##..|.GG....####"
        "##..############"
        "##..############"
        "##..############"
        "##|#############"
        "##|#############"
        "##|#############"
        ,Up},
        {
        "................"
        "................"
        "................"
        "................"
        "##.............."
        "##.............."
        "##.............."
        "##.............."
        "##.............."
        "##.............."
        "##.............."
        "##.............."
        "##............##"
        "###|############"
        "###|############"
        "###|############"
        ,Right},
        {
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "......##########"
        "......##########"
        "..##############"
        "..##############"
        "################"
        "################"
        "################"
        "################"
        "################"
        "################"
        ,Right},
        {
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "############...."
        "############...."
        "##.............."
        "##.............."
        "############|#.."
        "############|#.."
        "############|#.."
        "############|#.."
        ,Down},
        {
        "######......|..."
        "#####.......|..."
        "#####.......|..."
        "######|#######.."
        "######|#######.."
        "####............"
        "####............"
        "##.............."
        "##.............."
        "##....##..##...."
        "##.............."
        "##.............."
        "##.............."
        "######|#........"
        "######|#........"
        "######|#........"
        ,Down},
        {
        "######|#........"
        "######|#........"
        "######.........."
        "######.......^^."
        "########.....##."
        "########........"
        "####............"
        "####............"
        "##.............."
        "##.....##....##."
        "##.............."
        "##.............."
        "##.............."
        "##|#####^^^^^^^^"
        "##|#############"
        "##|#############"
        ,Down},
        {
        "##|............."
        "##|............."
        "##|............."
        "##|............."
        "##.............."
        "##.............."
        "##.............."
        "##.............."
        "##GGGG.........."
        "##GGGG.........."
        "####GGGG........"
        "####GGGG........"
        "################"
        "################"
        "################"
        "################"
        ,Right},
        {
        ".............###"
        ".............###"
        ".............###"
        ".............###"
        ".............###"
        ".............###"
        "................"
        "................"
        "................"
        "................"
        "..........######"
        "..........######"
        "################"
        "################"
        "################"
        "################"
        ,Shutter},
        {
        "................"
        "................"
        "................"
        "................"
        "################"
        "################"
        "................"
        "................"
        "................"
        "................"
        "################"
        "################"
        "................"
        "................"
        "................"
        "................"
        ,Right},
        {
        "................"
        "................"
        "................"
        "................"
        "################"
        "################"
        "................"
        "................"
        "................"
        "................"
        "################"
        "################"
        "................"
        "................"
        "................"
        "................"
        ,Right},
        {
        "................"
        "................"
        "................"
        "................"
        "################"
        "################"
        "................"
        "................"
        "................"
        "................"
        "################"
        "################"
        "................"
        "................"
        "................"
        "................"
        ,Shutter},
        {
        "################"
        "################"
        "#..............#"
        "#..............#"
        "#..............#"
        "#..............#"
        "...............#"
        "...............#"
        "...............#"
        "...............#"
        "##GGGG.........#"
        "##GGGG.........#"
        "################"
        "################"
        "################"
        "################"
        ,End}
    },

    // actives
    {
        Active( 2, 0, 128,64, 160,96 ),
        Active( 2, 0, 128,160, 160,192 ),
        Active( 2, 0, 160,128, 192,160 ),
        Active( 2, 0, 160,160, 192,192 ),
        Active( 2, 1, 128,64, 160,96 ),
        Active( 2, 1, 128,160, 160,192 ),
        Active( 2, 1, 160,128, 192,160 ),
        Active( 2, 1, 160,160, 192,192 ),
        Active( 2, 2, 128,96, 160,128 ),
        Active( 2, 8, 128,96, 160,128 ),
        Active( 2, 10, 96,160, 128,192 ),
        Active( 2, 11, 32,128, 64,160 ),
        Active( 2, 12, 96,128, 128,160 ),
        Active( 2, 18, 32,128, 64,160 ),
        Active( 2, 18, 64,128, 96,160 ),
        Active( 2, 18, 64,160, 96,192 ),
        Active( 2, 18, 96,160, 128,192 ),
        Active( 3, 19, 224,96, 0,160 ),
        Active( 1, 20, 0,96, 16,160 ),
        Active( 3, 22, 240,96, 0,160 ),
        Active( 2, 23, 32,160, 64,192 ),
        Active( 2, 23, 64,160, 96,192 ),
        Active( 1, 23, 0,96, 16,160 ),
        Active( 0,255, 0,0,0,0 )
    },

    // enemies
    {
        Enemy( 2, 16,76, 0 ),
        Enemy( 2, 64,140, 0 ),
        Enemy( 2, 80,92, 0 ),
        Enemy( 2, 56,76, 0 ),
        Enemy( 2, 112,140, 0 ),
        Enemy( 2, 128,92, 0 ),
        Enemy( 3, 56,72, 11 ),
        Enemy( 3, 184,120, 11 ),
        Enemy( 4, 120,88, 11 ),
        Enemy( 4, 152,56, 34 ),
        Enemy( 4, 216,152, 11 ),
        Enemy( 5, 72,184, 23 ),
        Enemy( 5, 120,120, 11 ),
        Enemy( 5, 184,56, 34 ),
        Enemy( 6, 120,216, 23 ),
        Enemy( 6, 152,40, 11 ),
        Enemy( 6, 184,88, 34 ),
        Enemy( 7, 144,180, 28 ),
        Enemy( 8, 56,184, 2 ),
        Enemy( 8, 88,152, 2 ),
        Enemy( 8, 120,120, 2 ),
        Enemy( 9, 72,168, 4 ),
        Enemy( 9, 104,104, 4 ),
        Enemy( 9, 184,136, 33 ),
        Enemy( 10, 72,72, 33 ),
        Enemy( 10, 88,136, 4 ),
        Enemy( 10, 120,104, 33 ),
        Enemy( 10, 216,136, 33 ),
        Enemy( 11, 120,168, 33 ),
        Enemy( 11, 136,184, 4 ),
        Enemy( 11, 184,104, 33 ),
        Enemy( 11, 216,72, 33 ),
        Enemy( 12, 40,72, 33 ),
        Enemy( 12, 56,120, 4 ),
        Enemy( 12, 56,168, 33 ),
        Enemy( 12, 136,136, 33 ),
        Enemy( 12, 200,40, 4 ),
        Enemy( 13, 144,180, 28 ),
        Enemy( 15, 40,184, 64 ),
        Enemy( 15, 96,76, 0 ),
        Enemy( 15, 144,140, 0 ),
        Enemy( 15, 160,92, 0 ),
        Enemy( 16, 248,128, 10 ),
        Enemy( 17, 248,128, 10 ),
        Enemy( 18, 224,172, 9 ),
        Enemy( 20, 216,160, 5 ),
        Enemy( 21, 200,96, 22 ),
        Enemy( 22, 80,160, 5 ),
        Enemy( 255, 0,0, 0 )
    },
    // palette
    {
        0x0F, 0x30, 0x2B, 0x06, 
        0x0F, 0x30, 0x2B, 0x2C, 
        0x0F, 0x20, 0x10, 0x00, 
        0x0F, 0x20, 0x10, 0x2C, 
        
        0x0F, 0x2C, 0x11, 0x0F, 
        0x0F, 0x20, 0x38, 0x0F, 
        0x0F, 0x30, 0x11, 0x0F, 
        0x0F, 0x30, 0x15, 0x0F, 
        
        0x0F, 0x30, 0x2B, 0x06, 
        0x0F, 0x30, 0x2B, 0x2C, 
        0x0F, 0x30, 0x20, 0x10, 
        0x0F, 0x20, 0x10, 0x2C, 
    },

    
    CutmanAscii,
    CutmanChar
};
