/*
ti pa

61.00 = wall
60.00 = walkable surface
65.00 = torch (right)
6E.00 = torch (up)
77.00 = wall detail (best used in horizontal stretches)


28.02 = background: red
29.02 = background: red - shadow
22.02 = background: red - window
23.02 = background: red - window shadow
2B.02 = background: red - window hole

00.00 = background: black

26.02 {27.02} = detail (black background)
28.02         = detail in detail

E4.00 = background: fence

83.00 = ladder

30.03 = lava surface
38.03 = lava

00.03 36.03
3F.03 3E.03 = lava flow corner /

6C.00 6D.00 = vert lava tunnel

74.00 75.00
7C.00 7D.00 = lava flow junction

73.00
7B.00 = horiz lava tunnel

51.00 52.00 53.00 = wily ornament
10.00 = wily door




*/
static void FiremanAscii(Screen& s)
{
    for(unsigned y=0; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
        {
            unsigned char c = s.outmap[y*16+x].tile;
            char ch = '?';
            /* Classify types.
             *
             *
             *
             */
            switch(c)
            {
                // air:
                case 0x00: // black bg
                case 0x26 ... 0x27: // detail
                case 0x2F: // detail in detail
                    ch = '.';
                    break;
                
                case 0xE4: // fence
                    ch = '=';
                    break;

                case 0x28:
                case 0x29:
                case 0x22:
                case 0x23:
                case 0x2B: // red walls
                    ch = ' ';
                    break;

                case 0x61: // wall
                case 0x60: // walkable wall
                case 0x51: case 0x52: case 0x53: // wily signature
                case 0x77: // wall detail
                    ch = '#';
                    break;
                
                case 0x6C: case 0x6D: // lava <->
                case 0x74: case 0x75: 
                case 0x7C: case 0x7D: // lava junction
                case 0x73: case 0x7B: // lava up/down
                    ch = '%';
                    break;
                
                case 0x36:
                case 0x3F: case 0x3E: // lava flow corner
                    ch = '/';
                    break;
                
                case 0x83:
                    ch = Ascii_Ladder;
                    break;
                
                case 0x10:
                    ch = Ascii_WilyDoor;
                    break;
                
                case 0x30:
                case 0x38:
                    ch = Ascii_Spike; // really lava
                    break;
                
                case 0x65: // torch right
                    ch = '>';
                    break;
                case 0x6E: // torch up
                    ch = 'T';
                    break;
                // no spikes or gutsblocks
            }
            
            s.ascii[y*32+x] = ch;
            s.ascii[y*32+x+16] = ' ';
            //printf("%02X",c);
        }
}

#define FiremanChar  DummyChar

static Level Fireman =
{
    3, // level number.
    "Fireman",

    // screens:
    {
        {
        "################"
        "################"
        "...............#"
        "...............#"
        "...............#"
        "...............#"
        "...............#"
        "...............#"
        "...............#"
        "...............#"
        "################"
        "################"
        "################"
        "################"
        "################"
        "################"
        ,End},
        {"",Last}
    },

    // actives
    {
    },

    // enemies
    {
    },
    
    // palette
    {
        0x0F, 0x20, 0x00, 0x02, 
        0x0F, 0x2B, 0x19, 0x2C, 
        0x0F, 0x17, 0x07, 0x0F, 
        0x0F, 0x20, 0x00, 0x26, 
        
        0x0F, 0x2C, 0x11, 0x0F, 
        0x0F, 0x20, 0x38, 0x0F, 
        0x0F, 0x27, 0x15, 0x0F, 
        0x0F, 0x30, 0x11, 0x0F, 
        
        0x0F, 0x20, 0x00, 0x02, 
        0x0F, 0x00, 0x10, 0x20, 
        0x0F, 0x17, 0x07, 0x07, 
        0x0F, 0x20, 0x00, 0x26, 
    },

    FiremanAscii,
    FiremanChar
};
