/*

ti pa

30.01 = 1st row cloud

32.01
01.01 = 2-3 row background

31.01 {30.01} 31.01
33.01 {34.01} 35.01 = 2-3 row cloud

01.01 = other row background

33.01 {34.01} 35.01 = other row cloud

3F.02 = frozen grass

3E.02 = frozen tree |

38.02 39.02 3A.02 
3B.02 3C.02 3D.02 = frozen tree leaves

A0.00 A1.00 = pillar surface
68.00 69.00 = pillar

77.00 = horizontal metal bar

A2.00 = vertical metal bar surface
6A.00 = vertical metal bar
6B.00 = vertical metal bar detail

66.00 67.00
6E.00 6F.00
76.00 6F.00 = metal object

E3.03 = water surface
C1.03 = water

2D.03 = inside
2C.03 = inside shadow

24.00 25.00 = plastic pit (background)

*/

static void IcemanAscii(Screen& s)
{
    for(unsigned y=0; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
        {
            unsigned char c = s.outmap[y*16+x].tile;
            unsigned char p = s.outmap[y*16+x].palette;
            
            char ch = '?';
            /* Classify types.
             *
             *
             *
             */
            switch(c)
            {
                // sky:
                case 0x32:
                case 0x01:
                case 0x31: case 0x30:
                case 0x33: case 0x34: case 0x35: // clouds, sky.
                case 0x00: //void
                case 0x3E: case 0x3F:
                case 0x38 ... 0x3A:
                case 0x3B ... 0x3D: // trees and grass
                    ch = '.';
                    break;
                
                case 0xA0: case 0xA1:
                case 0x68: case 0x69:
                case 0xA2: case 0x6A: case 0x6B: // walls
                case 0x51: case 0x52: case 0x53: // wily signature
                    ch = '#';
                    break;
                case 0x77:
                    ch = '=';
                    break;
                case 0x66: case 0x67:
                case 0x6E: case 0x6F:
                case 0x76: // metal object
                    ch = 'M';
                    break;
                case 0x2C: case 0x2D:
                    ch = ' '; // inside
                    break;
                case 0xC1: case 0xE3:
                    ch = '~'; // water
                    break;
                case 0x24: case 0x25:
                    ch = 'V';
                    break;
                    
                case 0x10:
                    ch = Ascii_WilyDoor;
                    break;
                
                // There are no ladders, gutsblock or spikes in Iceman level.
            }
            
            s.ascii[y*32+x] = ch;
            s.ascii[y*32+x+16] = ' ';
            //printf("%02X",c);      
        }
}

#define IcemanChar  DummyChar

static Level Iceman =
{
    1, // level number.
    "Iceman",

    // screens:
    {
        {
        "################"
        "################"
        "...............#"
        "...............#"
        "...............#"
        "...............#"
        "...............#"
        "...............#"
        "...............#"
        "...............#"
        "################"
        "################"
        "################"
        "################"
        "################"
        "################"
        ,End},
        {"",Last}
    },

    // actives
    {
    },

    // enemies
    {
    },

    // palette
    {
        0x0F, 0x20, 0x00, 0x02, 
        0x0F, 0x2B, 0x19, 0x2C, 
        0x0F, 0x17, 0x07, 0x0F, 
        0x0F, 0x20, 0x00, 0x26, 
        
        0x0F, 0x2C, 0x11, 0x0F, 
        0x0F, 0x20, 0x38, 0x0F, 
        0x0F, 0x27, 0x15, 0x0F, 
        0x0F, 0x30, 0x11, 0x0F, 
        
        0x0F, 0x20, 0x00, 0x02, 
        0x0F, 0x00, 0x10, 0x20, 
        0x0F, 0x17, 0x07, 0x07, 
        0x0F, 0x20, 0x00, 0x26, 
    },
    
    IcemanAscii,
    IcemanChar
};
