/*

ti pa

01.01 = outside sky
27.01 = grass
66.00 = outside floor

48.03 {49.03} 4A.03
50.03 {51.03} 52.03 = inside block type 1

4B.00 {4C.00} 4D.00
53.00 {54.00} 55.00 = inside block type 2

4E.03 = inside floor

38.00 39.00
3A.00 3B.00 = gutsblock

17.02 = inside background
16.02 = inside shadow

C5.00 = ladder

00.00 = black background

00.01 = boss room background

AE.00 = spike up (black bg)
BF.00 = spike down (black bg)

*/
static void Wily1Ascii(Screen& s)
{
    for(unsigned y=0; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
        {
            unsigned char c = s.outmap[y*16+x].tile;
            unsigned char p = s.outmap[y*16+x].palette;
            
            char ch = '?';
            /* Classify types.
             *
             *
             *
             */
            switch(c)
            {
                // air:
                case 0x00:
                    if(p == 1)
                        ch = 'X'; // boss room
                    else
                        ch = 'V'; // suspense
                    break;
                case 0x01:
                    ch = '.'; // sky
                    break;
                
                case 0x16:
                case 0x17: // inside
                case 0x27: // grass
                    ch = ' ';
                    break;
              
                case 0x48 ... 0x4A:
                case 0x50 ... 0x52: // block 1
                case 0x4B ... 0x4D:
                case 0x53 ... 0x55: // block 2
                case 0x66: // floor out
                case 0x4E: // floor in
                    ch = '#';
                    break;

                case 0xAE: //up spike
                case 0xBF: //down spike
                    ch = Ascii_Spike;
                    break;
                case 0xC5:
                    ch = Ascii_Ladder;
                    break;
                case 0x38:
                case 0x39:
                case 0x3A:
                case 0x3B:
                    ch = Ascii_Guts;
                    break;
            }
            
            s.ascii[y*32+x] = ch;
            s.ascii[y*32+x+16] = ' ';
            //printf("%02X",c);
        }
}
static void Wily1Char(Screen& s)
{
    const char* inmap = s.ascii;
    Char* outmap = s.outmap;

    char decorations[16*16];
    for(unsigned y=0; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
        {
            char c;
            switch(std::rand() % 6)
            {
                case 0: c = 1; break;
                case 1: c = 2; break;
                default: c = 0; break;
            }
            decorations[y*16+x] = c;
        }


    // Convert to tiles
    for(unsigned y=0; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
        {
            unsigned char c   = 0;
            unsigned char pal = 0;
            
            /* Wily1 palettes:
             *   
             * 0 = solid
             * 1 = blacks
             * 2 = blues
             * 3 = blue,gray
             */
            
            switch(inmap[y*16+x])
            {
                case '.':
                    c   = 0x01; // background
                    pal = 0x01;
                    break;
                case ' ':
                    c   = 0x27; // grass
                    pal = 0x01;
                    break;
                case '#':
                    c   = 0x66; // step
                    pal = 0x00;
                    break;
                case Ascii_Ladder:
                    c   = 0xC5; // ladder
                    pal = 0x00;
                    break;
                case Ascii_Spike:
                    c   = 0xAE; // spike
                    pal = 0x01;
                    break;
                case Ascii_Guts:
                    c = 0x38 + (x&1) + (y&1)*2; // gutsblock
                    pal = 0x00;
                    break;
                default:
                    printf("? '%c'\n", inmap[y*16+x]);
            }
            
            outmap[y*16+x].tile    = c;
            outmap[y*16+x].palette = pal;
        }
}

static Level Wily1 =
{
    6, // level number,
    "Wily1",

    // screens:
    {
        {
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "                "
        "################"
        "################"
        "################"
        "################"
        ,Right},
        {
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "                "
        "################"
        "################"
        "################"
        "################"
        ,Right},
        {
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "                "
        "################"
        "################"
        "################"
        "################"
        ,Right},
        {
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "......####......"
        "......####......"
        "....########...."
        "    ########    "
        "################"
        "################"
        "################"
        "################"
        ,Right},
        {
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "................"
        "......####......"
        "......####......"
        "....########...."
        "    ########    "
        "################"
        "################"
        "################"
        "################"
        ,Right},
        {
        "..........######"
        "..........######"
        "..........######"
        "..........######"
        "..........######"
        "..........######"
        "..........######"
        "..........######"
        "..........GGGGGG"
        "..........GGGGGG"
        "..........GG##GG"
        "          GG##GG"
        "################"
        "################"
        "################"
        "################"
        ,Right},
        {
        "#############|##"
        "#############|##"
        "########### .|##"
        "########### .|##"
        "#########.  .###"
        "#########.  .###"
        "######.  .##.###"
        "######.  .##.###"
        "      .##.######"
        "      .##.######"
        "    ##.#########"
        "    ##.#########"
        "################"
        "################"
        "################"
        "################"
        ,Up},
        {
        "##|#############"
        "##|#############"
        "##|#####   #####"
        "##|#####   #####"
        "##|   ####    ##"
        "##|   ####    ##"
        "##|        ##|##"
        "##|        ##|##"
        "##           |##"
        "##   ##      |##"
        "##   ##      |##"
        "##   ##  #  #|##"
        "##^^^##^^#^^#|##"
        "#############|##"
        "#############|##"
        "#############|##"
        ,Up},
        {
        "################"
        "################"
        "################"
        "################"
        "################"
        "################"
        "##              "
        "##              "
        "##              "
        "##              "
        "##            ##"
        "##            ##"
        "##|#############"
        "##|#############"
        "##|#############"
        "##|#############"
        ,Right},
        {
        "################"
        "################"
        "        GGGG  ##"
        "        GGGG  ##"
        "############  ##"
        "############  ##"
        "              ##"
        "              ##"
        "    ############"
        "    ############"
        "  GGGGGG      ##"
        "  GGGGGG      ##"
        "#############|##"
        "#############|##"
        "#############|##"
        "#############|##"
        ,Down},
        {
        "##            ##"
        "##            ##"
        "##            ##"
        "##  ############"
        "##            ##"
        "##            ##"
        "##            ##"
        "##            ##"
        "####  #  #  # ##"
        "####^^#^^#^^# ##"
        "############# ##"
        "##            ##"
        "##            ##"
        "##|#############"
        "##|#############"
        "##|#############"
        ,Down},
        {
        "## #############"
        "## #############"
        "##  ^^^^^^^^^^^^"
        "##              "
        "##              "
        "##              "
        "##              "
        "##              "
        "##              "
        "##              "
        "##              "
        "##              "
        "####            "
        "####^^^^^^^^^^^^"
        "################"
        "################"
        ,Right},
        {
        "######### ###|##"
        "######### ###|##"
        "^^^^^^^^^    |##"
        "             |##"
        "             |##"
        "             |##"
        "             |##"
        "             |##"
        "             |##"
        "             |##"
        "             |##"
        "             |##"
        "          ######"
        "^^^^^^^^^^######"
        "################"
        "################"
        ,Up},
        {
        "##|#############"
        "##|#############"
        "##|   ##      ##"
        "##|   ##      ##"
        "##    ##  ##  ##"
        "##    ##  ##  ##"
        "##    ##  ##  ##"
        "##    ##  ##  ##"
        "##    ##  ##  ##"
        "##    ##  ##  ##"
        "##        ##  ##"
        "##        ##  ##"
        "##        ##  ##"
        "#########.###|##"
        "#########.###|##"
        "#########.###|##"
        ,Up},
        {
        "################"
        "################"
        "################"
        "################"
        "################"
        "################"
        "##              "
        "##              "
        "##              "
        "##              "
        "##            ##"
        "##            ##"
        "##|#############"
        "##|#############"
        "##|#############"
        "##|#############"
        ,Shutter},
        {
        "################"
        "################"
        "...............#"
        "...............#"
        "...............#"
        "...............#"
        "...............#"
        "...............#"
        "...............#"
        "...............#"
        "################"
        "################"
        "################"
        "################"
        "################"
        "################"
        ,End}
    },

    // actives
    {
        Active( 2, 29, 160,128, 192,160 ),
        Active( 2, 29, 160,160, 192,192 ),
        Active( 2, 29, 192,128, 224,160 ),
        Active( 2, 29, 224,128, 0,160 ),
        Active( 2, 29, 224,160, 0,192 ),
        Active( 1, 30, 96,224, 112,224 ),
        Active( 1, 30, 144,192, 160,192 ),
        Active( 1, 30, 192,160, 208,160 ),
        Active( 2, 33, 32,160, 64,192 ),
        Active( 2, 33, 64,160, 96,192 ),
        Active( 2, 33, 96,160, 128,192 ),
        Active( 2, 33, 128,32, 160,64 ),
        Active( 2, 33, 160,32, 192,64 ),
        Active( 1, 39, 0,32, 16,192 ),
        Active( 0,255, 0,0,0,0 ),
    },

    // enemies
    {
        Enemy( 25, 8,172, 9 ),
        Enemy( 26, 248,172, 9 ),
        Enemy( 28, 112,108, 9 ),
        Enemy( 30, 104,224, 15 ),
        Enemy( 30, 152,192, 15 ),
        Enemy( 30, 200,160, 15 ),
        Enemy( 31, 144,56, 64 ),
        Enemy( 32, 112,168, 2 ),
        Enemy( 32, 176,168, 2 ),
        Enemy( 32, 240,168, 2 ),
        Enemy( 33, 8,56, 64 ),
        Enemy( 33, 56,56, 2 ),
        Enemy( 33, 120,56, 2 ),
        Enemy( 33, 216,120, 2 ),
        Enemy( 34, 240,128, 8 ),
        Enemy( 35, 228,172, 59 ),
        Enemy( 36, 48,172, 19 ),
        Enemy( 36, 132,172, 19 ),
        Enemy( 36, 184,188, 61 ),
        Enemy( 36, 200,188, 61 ),
        Enemy( 128, 0,0, 0 ),
        Enemy( 255, 0,0, 0 )
    },
    
    // palette
    {
        0x0F, 0x20, 0x00, 0x02, 
        0x0F, 0x2B, 0x19, 0x2C, 
        0x0F, 0x17, 0x07, 0x0F, 
        0x0F, 0x20, 0x00, 0x26, 
        
        0x0F, 0x2C, 0x11, 0x0F, 
        0x0F, 0x20, 0x38, 0x0F, 
        0x0F, 0x27, 0x15, 0x0F, 
        0x0F, 0x30, 0x11, 0x0F, 
        
        0x0F, 0x20, 0x00, 0x02, 
        0x0F, 0x00, 0x10, 0x20, 
        0x0F, 0x17, 0x07, 0x07, 
        0x0F, 0x20, 0x00, 0x26, 
    },

    Wily1Ascii,
    Wily1Char
};
